/*
 * FirstScript.java
 *
 * Created on 11. August 2002, 11:17
 */

import javax.swing.*;
import java.io.*;

import de.jensbruhn.south.plugin.SouthPluginIF;
import de.jensbruhn.south.plugin.SouthPluginToolsIF;
import de.jensbruhn.south.plugin.PrcImageIF;
import de.jensbruhn.south.plugin.OpcodeIF;

/**
 *
 * @author  Jens Bruhn
 */
public class SouthPluginMain implements SouthPluginIF
{
    
    private SouthPluginToolsIF tools;
    private JEditorPane editorPane;
    
    private boolean reactOnBreakpoints = false;
    
    private String variable;
    private int varOffset;
    private String varRegister;
    private long varAddress;
           
    private StringBuffer htmlText = new StringBuffer();
    
    /** Creates a new instance of FirstScript */
    public SouthPluginMain()
    {
    }

    public String getName()
    {
        return "Example plugin: global variables";
    }
    
    public void init(SouthPluginToolsIF tools, JPanel myPanel)
    {
        this.tools = tools;
        this.editorPane = new JEditorPane();
        editorPane.setContentType("text/html");
        
        myPanel.setLayout(new BoxLayout(myPanel, BoxLayout.X_AXIS));
        myPanel.add(new JScrollPane(editorPane));
                        
        appendText("<table ALIGN=\"CENTER\"><tr><td ALIGN=\"CENTER\"><FONT COLOR=\"#0000FF\">", false);
        appendText("<tr><td ALIGN=\"CENTER\"><FONT COLOR=\"#0000FF\">This is just a small example plugin.</td></tr>", false);
        appendText("<tr><td ALIGN=\"CENTER\"><FONT COLOR=\"#0000FF\">It will show you the value of a global variable</td></tr>", false);
        appendText("<tr><td ALIGN=\"CENTER\"><FONT COLOR=\"#0000FF\">after every breakpoint/step. It will refresh the value</td></tr>", false);
        appendText("<tr><td ALIGN=\"CENTER\"><FONT COLOR=\"#0000FF\">after all register and memory changes (user interaction),</td></tr>", false);
        appendText("<tr><td ALIGN=\"CENTER\"><FONT COLOR=\"#0000FF\">which will change the global variable.</font></td></tr></table>", false);
        
        appendText("<p><i><font color=\"#00FF00\">plugin initialized</font></i>", true);
    }
    
    public void started()
    {
        appendText("<i><font color=\"#00FF00\">plugin started...</font></i>", true);
        
        if (tools.isPOSEbusy() == true)
            JOptionPane.showMessageDialog(editorPane, "Please set POSE into debug-mode and restart the plugin.");
        else
        {
            variable = JOptionPane.showInputDialog(editorPane, "Please enter variable name (\"-100(A5)\"):");
            varOffset = Integer.parseInt(variable.substring(0, variable.indexOf("(")));
            varRegister = variable.substring(variable.indexOf("(") + 1, variable.indexOf(")")).toLowerCase();
            
            varAddress = tools.getRegisterValue(varRegister) + varOffset;
            appendText(variable + " is at " + tools.toHex(varAddress, 8), true);
            
            showInfo();
        }
                
        reactOnBreakpoints = true;        
    }
    
    public void stopped()
    {
        appendText("<i><font color=\"#00FF00\">plugin stopped...</font></i>", true);
        reactOnBreakpoints = false;
    }
    
    public void executionStopped()
    {
        if (reactOnBreakpoints == true)
        {
            long currentAddress = tools.getNextOpcode().getAddress();
            appendText("<b>" + tools.getNextOpcode().formatOpcode() + "</b>", true);
                      
            showInfo();
        }
    }
    
    public void executionStarted()
    {
    }
    
    public void breakpointChanged()
    {
    }
    
    public void memoryChanged(long address, int newValue, int oldValue)
    {
        appendText("<font color=\"#FF000000\">User interaction, memory change: " +
               tools.toHex(address, 8) + " " +
               tools.toHex(oldValue, 2) + "->" + tools.toHex(newValue, 2) +
               "</font>", true);
        
        showInfo();
    }
    
    public void registerChanged(String register)
    {
        appendText("<font color=\"#FF000000\">User interaction, register change: " + 
               register.toUpperCase() + "</font>", true);
        
        showInfo();
    }
        
    private void showInfo()
    {
        StringBuffer info = new StringBuffer();
        info.append("@" + tools.toHex(varAddress, 8) + ": ");
        info.append("<b><font color=\"#0000FF\">" + tools.toHex(tools.getValueFromBytes(tools.getMemoryContent(varAddress, 4), 0, 4), 8) + "</font></b>");

        info.append(" current value of " + variable + ": ");
        long curAddress = tools.getRegisterValue(varRegister) + varOffset;
        info.append("<b><font color=\"#0000FF\">" + tools.toHex(tools.getValueFromBytes(tools.getMemoryContent(curAddress, 4), 0, 4), 8) + "</font></b>");

        appendText(info.toString(), true);
    }
    
    private void appendText(String text, boolean withBR)
    {
        htmlText.append(text);
        
        if (withBR == true)
            htmlText.append("<br>");
                
        editorPane.setText("<html>" + htmlText + "</html>");
        editorPane.setCaretPosition(editorPane.getDocument().getLength() - 1);
    }
    
}
